# adlint-exam-c_builtin examination package loader.
#
# Author::    Yutaka Yanoh <mailto:yanoh@users.sourceforge.net>
# Copyright:: Copyright (C) 2010-2013, OGIS-RI Co.,Ltd.
# License::   GPLv3+: GNU General Public License version 3 or later
#
# Owner::     Yutaka Yanoh <mailto:yanoh@users.sourceforge.net>

#--
#     ___    ____  __    ___   _________
#    /   |  / _  |/ /   / / | / /__  __/           Source Code Static Analyzer
#   / /| | / / / / /   / /  |/ /  / /                   AdLint - Advanced Lint
#  / __  |/ /_/ / /___/ / /|  /  / /
# /_/  |_|_____/_____/_/_/ |_/  /_/   Copyright (C) 2010-2013, OGIS-RI Co.,Ltd.
#
# This file is part of AdLint.
#
# AdLint is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# AdLint is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# AdLint.  If not, see <http://www.gnu.org/licenses/>.
#
#++

require "adlint"

require "adlint/exam/c_builtin/cc1_code"
require "adlint/exam/c_builtin/cc1_check"
require "adlint/exam/c_builtin/cc1_check_shima"
require "adlint/exam/c_builtin/cc1_metric"
require "adlint/exam/c_builtin/cpp_code"
require "adlint/exam/c_builtin/cpp_check"
require "adlint/exam/c_builtin/cpp_check_shima"
require "adlint/exam/c_builtin/ld_check"
require "adlint/exam/c_builtin/ld_metric"

module AdLint #:nodoc:
module Exam #:nodoc:
module CBuiltin #:nodoc:

  Catalog = ExaminationCatalog.new(__FILE__) do |cat|
    cat.name = "c_builtin"

    cat.major_version = MAJOR_VERSION
    cat.minor_version = MINOR_VERSION
    cat.patch_version = PATCH_VERSION
    cat.release_date  = RELEASE_DATE

    cat.examination_classes = [
      W0001_Cc1, W0001_Cpp,
      W0002,
      W0003,
      W0007,
      W0010,
      W0013,
      W0016,
      W0017,
      W0018,
      W0019,
      W0021,
      W0023,
      W0024,
      W0025,
      W0026,
      W0027,
      W0028,
      W0030,
      W0031,
      W0033,
      W0035,
      W0036,
      W0037,
      W0038,
      W0039,
      W0040,
      W0041,
      W0042,
      W0043,
      W0049,
      W0050,
      W0051,
      W0052,
      W0053,
      W0054,
      W0055,
      W0056,
      W0057,
      W0058,
      W0059,
      W0060,
      W0061,
      W0062,
      W0063,
      W0064,
      W0065,
      W0066,
      W0067,
      W0068,
      W0069,
      W0070,
      W0071,
      W0072,
      W0073,
      W0076,
      W0077,
      W0078,
      W0079,
      W0080,
      W0081,
      W0082,
      W0084,
      W0085,
      W0086,
      W0087,
      W0088,
      W0093,
      W0096,
      W0097,
      W0100,
      W0101,
      W0102,
      W0103,
      W0104,
      W0105,
      W0107,
      W0108,
      W0109,
      W0110,
      W0112,
      W0114,
      W0115,
      W0116,
      W0117,
      W0118,
      W0119,
      W0120,
      W0121,
      W0122,
      W0123,
      W0124,
      W0125,
      W0126,
      W0127,
      W0128,
      W0129,
      W0130,
      W0131,
      W0132,
      W0133,
      W0134,
      W0135,
      W0136,
      W0137,
      W0138,
      W0139,
      W0140,
      W0141,
      W0142,
      W0143,
      W0144,
      W0145,
      W0146,
      W0147,
      W0148,
      W0149,
      W0150,
      W0151,
      W0152,
      W0153,
      W0154,
      W0155,
      W0156,
      W0157,
      W0158,
      W0159,
      W0160,
      W0161,
      W0162,
      W0163,
      W0164,
      W0165,
      W0166,
      W0167,
      W0168,
      W0169,
      W0170,
      W0171,
      W0172,
      W0173,
      W0174,
      W0175,
      W0176,
      W0177,
      W0178,
      W0179,
      W0180,
      W0181,
      W0182,
      W0183,
      W0184,
      W0185,
      W0186,
      W0187,
      W0188,
      W0189,
      W0190,
      W0191,
      W0192,
      W0193,
      W0194,
      W0195,
      W0196,
      W0197,
      W0198,
      W0199,
      W0200,
      W0201,
      W0202,
      W0203,
      W0204,
      W0205,
      W0206,
      W0207,
      W0208,
      W0209,
      W0210,
      W0211,
      W0212,
      W0213,
      W0214,
      W0215,
      W0216,
      W0217,
      W0218,
      W0219,
      W0220,
      W0221,
      W0222,
      W0223,
      W0224,
      W0225,
      W0226,
      W0227,
      W0228,
      W0229,
      W0230,
      W0231,
      W0232,
      W0233,
      W0234,
      W0235,
      W0236,
      W0237,
      W0238,
      W0239,
      W0240,
      W0241,
      W0242,
      W0243,
      W0244,
      W0245,
      W0246,
      W0247,
      W0248,
      W0249,
      W0250,
      W0251,
      W0252,
      W0253,
      W0254,
      W0255,
      W0256,
      W0257,
      W0258,
      W0259,
      W0260,
      W0261,
      W0262,
      W0263,
      W0264,
      W0265,
      W0266,
      W0267,
      W0268,
      W0269,
      W0270,
      W0271,
      W0272,
      W0273,
      W0274,
      W0275,
      W0276,
      W0277,
      W0278,
      W0279,
      W0280,
      W0281,
      W0282,
      W0283,
      W0284,
      W0285,
      W0286,
      W0287,
      W0288,
      W0289,
      W0290,
      W0291,
      W0292,
      W0293,
      W0294,
      W0295,
      W0296,
      W0297,
      W0298,
      W0299,
      W0300,
      W0301,
      W0302,
      W0303,
      W0304,
      W0305,
      W0306,
      W0307,
      W0308,
      W0309,
      W0310,
      W0311,
      W0312,
      W0313,
      W0314,
      W0315,
      W0316,
      W0317,
      W0318,
      W0319,
      W0320,
      W0321,
      W0322,
      W0323,
      W0324,
      W0325,
      W0326,
      W0327,
      W0328,
      W0329,
      W0330,
      W0331,
      W0332,
      W0333,
      W0334,
      W0335,
      W0336,
      W0337,
      W0338,
      W0339,
      W0340,
      W0341,
      W0342,
      W0343,
      W0344,
      W0345,
      W0346,
      W0347,
      W0348,
      W0349,
      W0350,
      W0351,
      W0352,
      W0353,
      W0354,
      W0355,
      W0356,
      W0357,
      W0358,
      W0359,
      W0360,
      W0361,
      W0362,
      W0363,
      W0364,
      W0365,
      W0366,
      W0367,
      W0368,
      W0369,
      W0370,
      W0371,
      W0372,
      W0373,
      W0374,
      W0375,
      W0376,
      W0377,
      W0378,
      W0379,
      W0380,
      W0381,
      W0382,
      W0383,
      W0384,
      W0385,
      W0386,
      W0387,
      W0388,
      W0389,
      W0390,
      W0391,
      W0392,
      W0393,
      W0394,
      W0395,
      W0396,
      W0397,
      W0398,
      W0399,
      W0400,
      W0401,
      W0402,
      W0403,
      W0404,
      W0405,
      W0406,
      W0407,
      W0408,
      W0409,
      W0410,
      W0411,
      W0413,
      W0414,
      W0421,
      W0422,
      W0423,
      W0424,
      W0425,
      W0431,
      W0432,
      W0440,
      W0441,
      W0442,
      W0443,
      W0444,
      W0445,
      W0446,
      W0447,
      W0456,
      W0457,
      W0458,
      W0459,
      W0460,
      W0461,
      W0462,
      W0477,
      W0478,
      W0479,
      W0480,
      W0481,
      W0482,
      W0483,
      W0488,
      W0489,
      W0490,
      W0491,
      W0492,
      W0493,
      W0495,
      W0496,
      W0497,
      W0498,
      W0499,
      W0500,
      W0501,
      W0502,
      W0508,
      W0511,
      W0512,
      W0525,
      W0528,
      W0529,
      W0530,
      W0532,
      W0534,
      W0535,
      W0538,
      W0540,
      W0541,
      W0542,
      W0543,
      W0544,
      W0546,
      W0549,
      W0551,
      W0552,
      W0553,
      W0554,
      W0555,
      W0556,
      W0559,
      W0560,
      W0561,
      W0562,
      W0563,
      W0564,
      W0565,
      W0566,
      W0567,
      W0568,
      W0569,
      W0570,
      W0571,
      W0572,
      W0573,
      W0574,
      W0575,
      W0576,
      W0577,
      W0578,
      W0579,
      W0580,
      W0581,
      W0582,
      W0583,
      W0584,
      W0585,
      W0586,
      W0589,
      W0591,
      W0593,
      W0597,
      W0598,
      W0599,
      W0600,
      W0605,
      W0606,
      W0607,
      W0608,
      W0609,
      W0610,
      W0611,
      W0612,
      W0613,
      W0614,
      W0622,
      W0623,
      W0624,
      W0625,
      W0626,
      W0627,
      W0628,
      W0629,
      W0632,
      W0633,
      W0634,
      W0635,
      W0636,
      W0637,
      W0638,
      W0639,
      W0640,
      W0641,
      W0642,
      W0643,
      W0644,
      W0645,
      W0646,
      W0649,
      W0650,
      W0653,
      W0654,
      W0655,
      W0656,
      W0657,
      W0658,
      W0659,
      W0660,
      W0661,
      W0662,
      W0663,
      W0664,
      W0665,
      W0666,
      W0667,
      W0668,
      W0669,
      W0670,
      W0671,
      W0672,
      W0673,
      W0674,
      W0675,
      W0676,
      W0677,
      W0678,
      W0679,
      W0680,
      W0681,
      W0682,
      W0683,
      W0684,
      W0685,
      W0686,
      W0687,
      W0688,
      W0689,
      W0690,
      W0691,
      W0692,
      W0694,
      W0695,
      W0696,
      W0697,
      W0698,
      W0699,
      W0700,
      W0703,
      W0704,
      W0705,
      W0707,
      W0708,
      W0711,
      W0712,
      W0713,
      W0714,
      W0715,
      W0716,
      W0717,
      W0718,
      W0719,
      W0720,
      W0721,
      W0722,
      W0723,
      W0726,
      W0727,
      W0728,
      W0729,
      W0730,
      W0731,
      W0732,
      W0733,
      W0734,
      W0735,
      W0736,
      W0737,
      W0738,
      W0739,
      W0740,
      W0741,
      W0742,
      W0743,
      W0744,
      W0745,
      W0747,
      W0748,
      W0749,
      W0750,
      W0751,
      W0752,
      W0753,
      W0754,
      W0755,
      W0756,
      W0757,
      W0758,
      W0759,
      W0760,
      W0761,
      W0762,
      W0763,
      W0764,
      W0765,
      W0766,
      W0767,
      W0768,
      W0769,
      W0770,
      W0771,
      W0774,
      W0775,
      W0776,
      W0777,
      W0778,
      W0779,
      W0780,
      W0781,
      W0783,
      W0785,
      W0786,
      W0787,
      W0788,
      W0789,
      W0790,
      W0791,
      W0792,
      W0793,
      W0794,
      W0795,
      W0796,
      W0797,
      W0798,
      W0799,
      W0800,
      W0801,
      W0804,
      W0805,
      W0806,
      W0807,
      W0808,
      W0809,
      W0810,
      W0811,
      W0827,
      W0828,
      W0830,
      W0831,
      W0832,
      W0833,
      W0834,
      W0947,
      W0948,
      W0949,
      W0950,
      W1026,
      W1027,
      W1028,
      W1029,
      W1030,
      W1031,
      W1032,
      W1033,
      W1034,
      W1037,
      W1039,
      W1040,
      W1041,
      W1046,
      W1047,
      W1049,
      W1050,
      W1051,
      W1052,
      W1053,
      W1054,
      W1055,
      W1056,
      W1057,
      W1058,
      W1059,
      W1060,
      W1061,
      W1062,
      W1063,
      W1064,
      W1065,
      W1066,
      W1067,
      W1068,
      W1069,
      W1070,
      W1071,
      W1072,
      W1073,
      W1074,
      W1075,
      W1076,
      W1077,
      W9001,
      W9002,
      W9003,

      FL_STMT,
      FL_FUNC,
      FN_STMT,
      FN_UNRC,
      FN_LINE,
      FN_PARA,
      FN_UNUV,
      FN_CSUB,
      FN_GOTO,
      FN_RETN,
      FN_UELS,
      FN_NEST,
      FN_PATH,
      FN_CYCM,
      FN_CALL,

      TypeDeclExtraction,
      GVarDeclExtraction,
      FunDclExtraction,
      VarDefExtraction,
      FunDefExtraction,
      LabelDefExtraction,
      InitializationExtraction,
      AssignmentExtraction,
      FuncallExtraction,
      XRefExtraction,
      LiteralExtraction,
      ObjLikeMacroExtraction,
      FunLikeMacroExtraction,
      IncludeDirectiveExtraction,
      DirectiveExtraction
    ]
  end

end
end
end
